/*-------------------------------------------------------------------------+
| Copyright 2017 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.common.ui.javafx;

import java.io.IOException;
import java.net.URL;

import javafx.application.Platform;
import javafx.fxml.FXMLLoader;
import javafx.scene.Scene;
import javafx.scene.layout.Pane;

import org.apache.commons.lang3.SystemUtils;
import org.eclipse.fx.ui.workbench3.FXViewPart;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;

/**
 * Base class to create eclipse (SWT-based) views using JavaFX GUI elements. This class requires
 * FXML file that defines the GUI elements of the view.
 * 
 * @author diewald
 */
/**
 * This class will be used for the migration to JavaFX, which is work-in-progress. Hence, mark the
 * class for non-removal.
 */
public abstract class AF3FXViewPart extends FXViewPart { // NO_UCD
	/** For later use, if we need to define different styles. */
	private String cssLocation;

	/** Flag that indicates if a custom style sheet has been applied. */
	private boolean isCustomStyleSheetApplied;

	/**
	 * Holds the relative location of the FXML JavaXML files. Roots are the source folders of the
	 * (plugin) project.
	 */
	private final String fXMLLocation;

	/** References the stub that defines the FXML file defining the layout and controller. */
	private Class<? extends AF3FXViewPart> viewerClass;

	/**
	 * Constructor. Allows to pass the view's FXML definition file along with style.
	 * 
	 * @param viewerClass
	 *            Stub class that defines the FXML file defining the layout and controller
	 * @param fXMLLocation
	 *            location of the fXML file relative to the source (resource) folders.
	 * @throws Exception
	 *             if the given FXML file path is not given or if it cannot be found in any of the
	 *             source (resource) folders.
	 */
	public AF3FXViewPart(Class<? extends AF3FXViewPart> viewerClass, String fXMLLocation)
			throws Exception {
		if(fXMLLocation == null) {
			throw new Exception(
					"The location of the FXML file defining the gui elements was not given," +
							"or it cannot be found at the given location (" + fXMLLocation + ") " +
							"The view will NOT be created.");
		}

		Platform.setImplicitExit(false);
		this.viewerClass = viewerClass;
		this.fXMLLocation = fXMLLocation;
	}

	/**
	 * Constructor. Allows to pass the view's FXML definition file.
	 * 
	 * @param viewerClass
	 *            Stub class that defines the FXML file defining the layout and controller
	 * @param fXMLLocation
	 *            location of the fXML file relative to the source (resource) folders.
	 * @param cssLocation
	 *            style sheet file path for the appearance settings relative to the source
	 *            (resource) folders.
	 * @throws Exception
	 *             if the given FXML file path is not given or if it cannot be found in any of the
	 *             source (resource) folders.
	 */
	public AF3FXViewPart(Class<? extends AF3FXViewPart> viewerClass, String fXMLLocation,
			String cssLocation) throws Exception {
		this(viewerClass, fXMLLocation);
		this.cssLocation = cssLocation;
	}

	/** {@inheritDoc} */
	@Override
	public void createPartControl(Composite parent) {
		if("gtk".equals(System.getProperty("osgi.ws"))) {
			if(!"0".equals(System.getenv("SWT_GTK3"))) {
				org.eclipse.swt.widgets.Label errorLabel =
						new org.eclipse.swt.widgets.Label(parent, SWT.BOLD);
				errorLabel
						.setText("Cannot show JavaFX view without setting SWT_GTK3 to '0'.\n"
								+ "Please relaunch Eclipse using a terminal and 'SWT_GTK3=0 ./eclipse' command.");
				return;
			}
		}
		super.createPartControl(parent);
	}

	/** {@inheritDoc} */
	@Override
	protected Scene createFxScene() {
		Pane root = null;
		FXMLLoader loader = new FXMLLoader();
		loader.setClassLoader(viewerClass.getClassLoader());
		loader.setLocation(viewerClass.getResource(fXMLLocation));
		try {
			root = (Pane)loader.load();
		} catch(IOException e) {
			throw new RuntimeException("Cannot load the resource located at " + fXMLLocation, e);
		}

		if(cssLocation != null) {
			isCustomStyleSheetApplied = applyStyleSheet(root, getClass(), cssLocation);
		}
		if(!isCustomStyleSheetApplied) {
			setNativeLook(root);
		}
		Scene scene = new Scene(root);
		return scene;
	}

	/** {@inheritDoc} */
	@Override
	protected void setFxFocus() {
		// Not needed.
	}

	/**
	 * Sets the native look of the target platform to break the UI. Most of the UI code is based on
	 * SWT which uses a platform native look. Currently, Windows 8&10, Windows 7, and MacOS X native
	 * looks are supported.
	 */
	private void setNativeLook(Pane pane) {
		if(SystemUtils.IS_OS_WINDOWS_10 || SystemUtils.IS_OS_WINDOWS_8) {
			applyStyleSheet(pane, AF3FXViewPart.class, "/jfxtras/JMetroLightTheme.css");
		} else if(SystemUtils.IS_OS_WINDOWS) {
			applyStyleSheet(pane, AF3FXViewPart.class, "/aerofx/win7.css");
		} else if(SystemUtils.IS_OS_MAC_OSX) {
			applyStyleSheet(pane, AF3FXViewPart.class, "/aquafx/aquafx.css");
		} else if(SystemUtils.IS_OS_LINUX) {
			// At the time of writing this method, no CSS is known that mimics GTK or QT. Thus, use
			// the default modena look.
		}
	}

	/**
	 * Applies the style sheet from the given location to the given {@link Pane}. The class loader
	 * of the plugin containing the style sheet must be given such that this resource can be loaded.
	 * For instance the class loader of the common.ui plugin must be given for the native looks.
	 * 
	 * @param pane
	 *            {@link Pane} to which the style sheet shall be applied.
	 * @param viewerClass
	 *            Class containing the style sheet resource.
	 * @param styleSheetLocation
	 *            relative location of the style sheet to its containing resource folder (aka source
	 *            folder).
	 * @return whether the style sheet could be applied.
	 */
	private boolean applyStyleSheet(Pane pane, Class<?> viewerClass, String styleSheetLocation) {
		if(viewerClass == null || styleSheetLocation == null) {
			return false;
		}
		URL cssURL = viewerClass.getResource(styleSheetLocation);
		if(cssURL != null) {
			pane.getStylesheets().add(cssURL.toExternalForm());
			return true;
		}
		return false;
	}
}
